from keras.models import load_model

import numpy


class Application:
    
    def __init__(self, model_path = None):
        """Initializes a neural network application.

        If you want to load an application you've already trained, just give a path of application file.
        
        Keyword Arguments:
            model_path {str} -- A path of application file. (default: {None})
        
        Raises:
            e -- Raises error when loading an application is failed.
        """
        if type(model_path) == type(None):
            self.model = None
        else:
            try:
                self.model = load_model(model_path)
                self.input_shape = self.model.input_shape[1:]
                if type(self.model) == type(None):
                    print('Nothing loaded')
            except Exception as e:
                print('Model loading failed')
                raise e

    def compile_model(self, optimizer, loss=None, metrics=['accuracy']):
        """This function compiles self.model according to arguments.
        
        Arguments:
            optimizer {str or keras optimizer instance} -- See https://keras.io/optimizers/ for more information about optimizers.
        
        Keyword Arguments:
            loss {str or objective function} -- See https://keras.io/losses/ for more information about loss. (default: {None})
            metrics {list} -- List of metrics to be evaluated by the application during training and evaluating(testing). (default: {['accuracy']})
        """
        self.model.compile(optimizer, loss=loss, metrics=metrics)  

    def evaluate(self, data_array, target_array, batch_size = 32, verbose = 1, sample_weight = None, steps = None):
        """Returns the loss value & metrics values for the application in test mode.
       
        Arguments:
            data_array {ndarray} -- The input data like x_test of keras.
            target_array {ndarray} -- Appropriate outputs of data_array.
        
        Keyword Arguments:
            batch_size {int} -- Number of samples per evaluation step. (default: {32})
            verbose {int} -- Verbosity mode. 0 = silent, 1 = progress bar. (default: {1})
            sample_weight {ndarray or None} --
                Optional Numpy array of weights for the test samples, used for weighting the loss function.
                Check https://keras.io/models/model/ for more information about sample_weight.
                (default: {None})
            steps {int or None} --
                Total number of steps (batches of samples) before declaring the evaluation round finished.
                Ignored with the default value of None.
                (default: {None})
        """
        evaluation_result = self.model.evaluate(data_array, target_array, batch_size = batch_size, verbose = verbose, sample_weight = sample_weight, steps = steps)
        return evaluation_result

    def predict(self, data_array, predict_classes = True, batch_size = 32, verbose = 0, steps = None):
        """Generates output of predictions for the input samples.
        
        Arguments:
            data_array {ndarray} -- The input data like x_test of keras.
        
        Keyword Arguments:
            predict_classes {bool} --
                Decides a prediction result's type.
                By default, a return value of prediction result is an array of classes.
                If you set predict_classes as False, you can get a raw output array as a prediction result.
                (default: {True})
            batch_size {int} -- Number of samples per prediction. (default: {32})
            verbose {int} -- Verbosity mode, 0 or 1.(default: {0})
            steps {int or None} --
                Total number of steps (batches of samples) before declaring the prediction round finished.
                Ignored with the default value of None.
                (default: {None})

        Raises:
            e -- Raise exception when prediction failed.
        
        Returns:
            [ndarray] -- Predicted classes(labels) of input data
        """
        try:
            probs = self.model.predict(data_array, batch_size = batch_size, verbose = verbose, steps = steps)
            if predict_classes == False:
                return probs
            predicted_classes = probs.argmax(axis=-1)
            try:
                for i in range(predicted_classes.shape[0]):
                    if i == 0:
                        return_list = [ self.model.output_dictionary[predicted_classes[i]] ]
                    else:
                        return_list.append( self.model.output_dictionary[predicted_classes[i]] )
                return numpy.asarray(return_list)
            except:
                return predicted_classes
        except Exception as e:
            # Prediction failed
            raise e

    def print_summary(self):
        self.model.summary()

    def save(self, model_path):
        """Saves an application as h5 format.
        
        Arguments:
            model_path {str} -- Path to save application.
                
        Raises:
            TypeError -- Raises error if self.model is not created.
            e -- Raises error if a path is wrong.
        """
        try:
            if type(self.model) == type(None):
                raise TypeError('you should create an application before save it')
            self.model.save(model_path)
        except Exception as e:
            raise e

    def set_output_dictionary(self, output_dictionary):
        self.model.output_dictionary = output_dictionary
        self.model.__class__.output_dictionary = self.model.output_dictionary 



from keras.applications import resnet50

from .image_classification import Image_Classification

class ResNet50(Image_Classification):
    def __init__(self, *args, **kwargs):
        Image_Classification.__init__(self, *args, **kwargs)
        if hasattr(self, 'input_shape') == False:
            self.input_shape = (224, 224, 3)
    
    def create_model(self, num_classes):
        model = resnet50.ResNet50( weights=None, input_shape=self.input_shape, classes=num_classes )
        return model


#-*- coding: utf-8 -*-
"""This module is a utility module for image preprocessing.
"""
import numpy

import cv2

import os

import sys



IMAGE_EXTENSIONS = set(['png', 'jpg', 'jpeg'])


def resize_image(img, target_size, cv_interpolation = cv2.INTER_LINEAR):
    """Resizes image with specified target size and casts to float32.
    
    Arguments:
        img {ndarray} -- An opencv image array
        target_size {tuple} -- A tuple of two natural numbers. Two numbers represent the width and height, respectively. 
    
    Keyword Arguments:
        cv_interpolation {int} -- cv2 interpolation. (default: {cv2.INTER_LINEAR})
    
    Returns:
        [ndarray] -- Resized image array.
    """
    resized_img = cv2.resize(img, target_size, interpolation = cv_interpolation)
    resized_img = resized_img.astype('float32')
    return resized_img

def is_image_file(filename):
    """this function checks a file extension.
    
    Arguments:
        filename {str} -- A path of file.
    
    Returns:
        [bool] -- If an extension of the file is an image, returns True. Otherwise False.
    """
    return '.' in filename and filename.rsplit('.', 1)[1] in IMAGE_EXTENSIONS

def get_current_directory_name(file_path):
    """This function returns a parent directory name of the file at the file_path.
    
    Arguments:
        file_path {str} -- A path of the file.
    
    Raises:
        ValueError -- If the file is not in a directory, then this function raises a ValueError.
    
    Returns:
        [str or unicode] -- A parent directory name.(The same as a label of the file.)
    """
    splitted_filename = None
    if sys.platform == "win32":
        splitted_filename = file_path.split('\\')
    else:
        splitted_filename = file_path.split('/')
    if len(splitted_filename) < 2:
        raise ValueError('No parent directory found. You should put your image files inside a directory name which is the same as a label of them.')
    else:
        return splitted_filename[-2]

def append_elem(path, img_list, label_list, target_size):
    """This function recursively iterates over files and directories.
    While iterating, if an image file found,
    this function appends an image as an array format to img_list and appends a label to label_list
    
    Arguments:
        path {str} -- A path of a current file or a director.
        img_list {list} -- A list of images.
        label_list {list} -- A list of labels.
        target_size {tuple} -- A tuple which specifies a target size of image array.(Image files are resized to target size.)
    """
    if os.path.isdir(path):
        child_list = os.listdir(path)
        for child in child_list:
            child_path = os.path.join(path, child)
            append_elem(child_path, img_list, label_list, target_size)
    else:
        if is_image_file(path):
            img = cv2.imread(path)
            resized_image = resize_image(img, target_size)
            label = get_current_directory_name(path)
            img_list.append(resized_image)
            label_list.append(label)

def directory_images_to_arrays(directory_path, target_size):
    """Recursively iterates over child files and directories of directory_path.
    Image files will be converted to an array of images with rescaled value.
    Parent directory names will be converted to array_of_lables.
    
    Arguments:
        directory_path {str} -- A root path of image files and directories.
        target_size {tuple} -- A tuple which specifies a target size of image array.(Image files are resized to target size.)
    
    Returns:
        array_of_images, array_of_labels [(ndarray, ndarray)] --
            array_of_images is a rescaled array of resized images.
            array_of_labels is a label array of images.
            You can find a label of an image at the same index of array_of_labels.
    """
    img_list = list()
    label_list = list()
    append_elem(directory_path, img_list, label_list, target_size)
    array_of_images = numpy.asarray(img_list)
    array_of_images /= 255
    array_of_labels = numpy.asarray(label_list)
    return array_of_images, array_of_labels